package PJVM::Runtime::Array;

use strict;
use warnings;

sub new {
    my ($pkg, $class, @dimensions) = @_;
    
    
}

1;
__END__

=head1 NAME

PJVM::Runtime::Array -

=head1 SYNOPSIS

=head1 DESCRIPTION

=head1 INTERFACE
