package PJVM::Runtime::Native::java::lang::Class;

use strict;
use warnings;

use PJVM::Runtime::Class;

sub new {
    my $class = PJVM::Runtime::Class->new(
        qname => "java/lang/Class",
        methods => {            
        },
    );
    
    return $class;
}

1;
__END__

=head1 NAME

PJVM::Runtime::Native::java::lang::Class -

=head1 SYNOPSIS

=head1 DESCRIPTION

=head1 INTERFACE
